#!/usr/bin/python
#
# parse json dbx from
#   https://github.com/microsoft/secureboot_objects/tree/main/PreSignedObjects/DBX
#
""" parse json dbx """
import sys
import json
import logging
import argparse

from virt.firmware.efi import guids
from virt.firmware.efi import efivar
from virt.firmware.efi import siglist

from virt.firmware.varstore import jstore

def hash_siglist(hashes):
    sl = siglist.EfiSigList(guid = guids.parse_str(guids.EfiCertSha256))
    guid = guids.parse_str(guids.MicrosoftVendor)
    for h in hashes:
        sl.add_sig(guid, bytes.fromhex(h.get('authenticodeHash')))
    sdb = siglist.EfiSigDB()
    sdb.append(sl)
    return sdb

def svns_siglist(svns):
    sl = siglist.EfiSigList(guid = guids.parse_str(guids.EfiCertSha256))
    guid = guids.parse_str(guids.MicrosoftVendor)
    for s in svns:
        sl.add_sig(guid, bytes.fromhex(s.get('value')))
    sdb = siglist.EfiSigDB()
    sdb.append(sl)
    return sdb

def cert_sigdb(certificates):
    for cert in certificates:
        # 'value' has a filename, that does not exist though
        cn = cert.get('subjectName')
        print(f'# cert    : "{cn}"  [NOT HANDLED]')
        logging.debug('certificate record: %s', cert)

def main():
    parser = argparse.ArgumentParser(
        description = "parse json dbx")
    parser.add_argument('-l', '--loglevel', dest = 'loglevel', type = str, default = 'info',
                        help = 'set loglevel to LEVEL', metavar = 'LEVEL')
    parser.add_argument('-i', '--input', dest = 'input', type = str,
                        help = 'read dbx revocation list from FILE', metavar = 'FILE')
    parser.add_argument('--print-varstore', dest = 'print_varstore',
                        action = 'store_true', default = False,
                        help = 'print varstore')
    parser.add_argument('--write-varstore', dest = 'write_varstore',
                        action = 'store_true', default = False,
                        help = 'write varstore (for "virt-fw-vars --set-json $file")')

    options = parser.parse_args()
    logging.basicConfig(format = '%(levelname)s: %(message)s',
                        level = getattr(logging, options.loglevel.upper()))

    with open(options.input, "r", encoding = 'utf-8') as f:
        dbx = json.loads(f.read())

    # debug
    logging.debug('toplevel keys: %s', dbx.keys())

    # handle image hashes
    hash_sdb = {}
    for (arch, data) in dbx.get('images').items():
        hash_sdb[arch] = hash_siglist(data)
        print(f'# {arch:8}: {len(hash_sdb[arch][0]):4} entries')

    # handle SVNs
    svns = dbx.get('svns')
    if svns:
        svns_sdb = svns_siglist(svns)
        print(f'# svns    : {len(svns_sdb[0]):4} entries')

    # handle certificates [TODO]
    certificates = dbx.get('certificates')
    if certificates:
        cert_sigdb(certificates)

    # write out as json varstore (usable via 'virt-fw-vars --set-json $file')
    for (arch, sdb) in hash_sdb.items():
        sdb.merge(svns_sdb)
        v = efivar.EfiVar('dbx', data = bytes(sdb))
        varlist = efivar.EfiVarList()
        varlist['dbx'] = v
        filename = f'dbx-{arch}.json'
        if options.write_varstore:
            jstore.JsonVarStore.write_varstore(filename, varlist)
        if options.print_varstore:
            varlist.print_normal()

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python
#
# SPDX-License-Identifier: GPL-2.0-only
# (c) 2023 Gerd Hoffmann
#
""" inspect secure boot variable updates """
import sys
import struct
import argparse

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.serialization import pkcs7, Encoding

from virt.firmware.efi import guids
from virt.firmware.efi import efivar
from virt.firmware.efi import siglist
from virt.firmware.misc import cert_not_valid_before, cert_not_valid_after

# UEFI spec allows pkcs7 signatures being used without the envelope
# which identifies them as pkcs7 signatures.  Most crypto libs will
# not parse them without the envelope though.  So add it if needed.
def wrap_pkcs7(data):
    wrap = bytes([
        0x06, 0x09,
        0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x07, 0x02,
        0xa0, 0x82,
    ])

    if wrap == data [ 4 : 17 ]:
        # envelope is present
        return data

    print('wrapping pkcs7 signature')
    ret = bytes([ 0x30, 0x82 ])
    ret += (len(data) + 15).to_bytes(length=2, byteorder='big')
    ret += wrap
    ret += len(data).to_bytes(length=2, byteorder='big')
    ret += data
    return ret

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--update', dest = 'update', type = str,
                        help = 'read sb variable update from FILE', metavar = 'FILE')
    parser.add_argument('--pem', dest = 'pem',
                        action = 'store_true', default = False,
                        help = 'print certificates in PEM format')
    options = parser.parse_args()

    if not options.update:
        print('missing kek update file (try --help)')
        return 1

    print('inspecting ' + options.update)
    with open(options.update, 'rb') as f:
        authdata = f.read()

    # parse struct EFI_TIME + EFI_VARIABLE_AUTHENTICATION_2
    (year, month, day, hour, minute, second, ns, tz, dl) = \
        struct.unpack_from("=HBBBBBxLhBx", authdata)
    print(f'time: {year:04}-{month:02}-{day:02} {hour:02}:{minute:02}:{second:02}')
    (length, revision, certtype) = struct.unpack_from("=LHH", authdata, 16)
    guid = guids.parse_bin(authdata, 24)
    if str(guid) != guids.EfiCertPkcs7:
        raise RuntimeError('no pkcs7 signature')
    cert_data = authdata [ 40 : 16 + length ]
    data = authdata [ 16 + length : ]

    if len(cert_data) == 0:
        print('pkcs7 signature is empty')
    else:
        certs = pkcs7.load_der_pkcs7_certificates(wrap_pkcs7(cert_data))
        print('pkcs7 signature chain certificates')
        for c in certs:
            print('  certificate')
            print('    fingerprint: ' + c.fingerprint(hashes.SHA256()).hex(':'))
            print('    subject    : ' + c.subject.rfc4514_string())
            print('    issuer     : ' + c.issuer.rfc4514_string())
            print('    valid from : ' + str(cert_not_valid_before(c)))
            print('    valid to   : ' + str(cert_not_valid_after(c)))
            if options.pem:
                print(c.public_bytes(Encoding.PEM).decode())

    sigdb = siglist.EfiSigDB(data)
    print('signature database')
    for slist in sigdb:
        efivar.EfiVarList.print_siglist(slist)

    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python
#
# based on pyuefivars which is:
#   Copyright Amazon.com, Inc.
#   SPDX-License-Identifier: MIT
#
###################################################################################
#
#
# variable store
#
# - ?
# - 'EFI_NV'    (6)
# - ?           (6)
# - 'VMWNVRAM'  (8)
# - ?           (4)
# - totalsize   (4)
#
#
# variable entry
#
# - guid        (16)
# - attributes  (4)
# - entrysize   (4)
# - namesize    (4)
# - name        (namesize)
# - data        (entrysize - namesize)
#
#
""" vmware nvram varstore parser """
import struct
import logging
import argparse

from virt.firmware.efi import guids
from virt.firmware.efi import ucs16
from virt.firmware.efi import efivar

from virt.firmware.varstore import jstore

class VmwareVarStore:
    """  class for vmware nvram varstore """

    def __init__(self, filename = None):
        self.filename = filename
        self.filedata = b''

        if self.filename:
            self.readfile()

    @staticmethod
    def probe(filename):
        with open(filename, "rb") as f:
            data = f.read()
        if b'VMWNVRAM' in data:
            return True
        return False

    def readfile(self):
        logging.info('reading vmware varstore from %s', self.filename)
        with open(self.filename, "rb") as f:
            self.filedata = f.read()

    def get_varlist(self):
        varlist = efivar.EfiVarList()
        start = self.filedata.find(b'VMWNVRAM')

        (unknown, totalsize) = struct.unpack_from('<LL', self.filedata, start + 8)
        logging.debug('totalsize=%d', totalsize)
        offset = 16

        while offset + 24 < totalsize:
            guid = guids.parse_bin(self.filedata, start + offset)
            (attr, entrysize, namesize) = struct.unpack_from('<LLL', self.filedata,
                                                             start + offset + 16)
            name = ucs16.from_ucs16(self.filedata, start + offset + 16 + 12)
            data = self.filedata [ start + offset + 16 + 12 + namesize :
                                   start + offset + 16 + 12 + entrysize ]
            logging.debug('  %-16s - attr=0x%02x entrysize=%d', name, attr, entrysize)

            if attr & 0x20:
                # 40 bytes additioal metadata, skip for now.
                var = efivar.EfiVar(name, guid = guid, attr = attr, data = data[40:])
            else:
                var = efivar.EfiVar(name, guid = guid, attr = attr, data = data)

            varlist[str(var.name)] = var
            offset += 16 + 12 + entrysize

        return varlist

    @staticmethod
    def write_varstore(filename, varlist):
        raise RuntimeError('writing vmware varstores is not supported')


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--input', dest = 'infile', type = str, default = "tests/data/esx7.nvram")
    parser.add_argument('--out-json', dest = 'outfile', type = str)
    options = parser.parse_args()

    logging.basicConfig(format = '%(levelname)s: %(message)s',
                        level = logging.DEBUG)

    if VmwareVarStore.probe(options.infile):
        vmwarestore = VmwareVarStore(options.infile)
        vl = vmwarestore.get_varlist()
        vl.print_normal()
        if options.outfile:
            jstore.JsonVarStore.write_varstore(options.outfile, vl)



#!/usr/bin/python
#
# SPDX-License-Identifier: GPL-2.0-only
# (c) 2023 Gerd Hoffmann
#
""" print and edit ovmf varstore files """
import sys
import json
import logging
import argparse

from virt.firmware.efi import guids
from virt.firmware.efi import efivar
from virt.firmware.efi import efijson
from virt.firmware.efi import devpath
from virt.firmware.efi import ucs16
from virt.firmware.efi import certs

from virt.firmware.varstore import aws
from virt.firmware.varstore import jstore
from virt.firmware.varstore import authfiles
from virt.firmware.varstore import autodetect


##################################################################################################
# main

# pylint: disable=too-many-branches,too-many-statements
def main():
    parser = argparse.ArgumentParser(
        description = "Print and modify EFI variable stores.")

    parser.add_argument('-l', '--loglevel', dest = 'loglevel', type = str, default = 'info',
                        help = 'set loglevel to LEVEL', metavar = 'LEVEL')
    parser.add_argument('-i', '--input', dest = 'input', type = str,
                        help = 'read edk2 or aws vars from FILE', metavar = 'FILE')
    parser.add_argument('--inplace', '--in-place', dest = 'inplace', type = str,
                        help = 'modify FILE in place', metavar = 'FILE')
    parser.add_argument('--extract-certs', dest = 'extract',
                        action = 'store_true', default = False,
                        help = 'extract all certificates')

    pgroup = parser.add_argument_group('Variable options')
    pgroup.add_argument('-d', '--delete', dest = 'delete',
                        action = 'append', type = str,
                        help = 'delete variable VAR, can be specified multiple times',
                        metavar = 'VAR')
    pgroup.add_argument('--set-true', dest = 'set_true',
                        action = 'append', type = str,
                        help = 'set variable VAR to true, can be specified multiple times',
                        metavar = 'VAR')
    pgroup.add_argument('--set-false', dest = 'set_false',
                        action = 'append', type = str,
                        help = 'set variable VAR to false, can be specified multiple times',
                        metavar = 'VAR')
    pgroup.add_argument('--set-json', dest = 'set_json', type = str,
                        help = 'set variables from json dump FILE',
                        metavar = 'FILE')

    pgroup = parser.add_argument_group('Boot configuration')
    pgroup.add_argument('--set-boot-uri', dest = 'set_boot_uri',
                        help = 'set network boot uri to LINK (once, using BootNext)',
                        metavar = 'LINK')
    pgroup.add_argument('--append-boot-filepath', dest = 'append_boot_filepath',
                        action = 'append', type = str,
                        help = 'append boot entry for FILE (permanent, using BootOrder)',
                        metavar = 'FILE')

    pgroup = parser.add_argument_group('shim.efi configuration')
    pgroup.add_argument('--set-shim-debug', dest = 'set_shim_debug',
                        action = 'store_true', default = False,
                        help = 'enable shim.efi debugging (pause for debugger attach)')
    pgroup.add_argument('--set-shim-verbose', dest = 'set_shim_verbose',
                        action = 'store_true', default = False,
                        help = 'enable shim.efi verbose messages')
    pgroup.add_argument('--set-fallback-verbose', dest = 'set_fallback_verbose',
                        action = 'store_true', default = False,
                        help = 'enable fallback.efi verbose messages')
    pgroup.add_argument('--set-fallback-no-reboot', dest = 'set_fallback_no_reboot',
                        action = 'store_true', default = False,
                        help = 'disable rebooting for fallback.efi')
    pgroup.add_argument('--set-sbat-level', dest = 'sbatlevel', type = str,
                        help = 'set SbatLevel variable', metavar = 'FILE')

    pgroup = parser.add_argument_group('Secure boot setup options')
    pgroup.add_argument('--set-pk', '--set-pk-cert', dest = 'pk',  nargs = 2,
                        help = 'set PK to x509 cert, loaded in pem or der format ' +
                        'from FILE and with owner GUID',
                        metavar = ('GUID', 'FILE'))
    pgroup.add_argument('--add-kek', '--add-kek-cert', dest = 'kek',  action = 'append', nargs = 2,
                        help = 'add x509 cert to KEK, loaded in pem or der format ' +
                        'from FILE and with owner GUID, can be specified multiple times',
                        metavar = ('GUID', 'FILE'))
    pgroup.add_argument('--add-db', '--add-db-cert', dest = 'db',  action = 'append', nargs = 2,
                        help = 'add x509 cert to db, loaded in pem or der format ' +
                        'from FILE and with owner GUID, can be specified multiple times',
                        metavar = ('GUID', 'FILE'))
    pgroup.add_argument('--add-db-hash', dest = 'db_hash',  action = 'append', nargs = 2,
                        help = 'add sha256 HASH to db, with owner GUID, ' +
                        'can be specified multiple times',
                        metavar = ('GUID', 'HASH'))
    pgroup.add_argument('--set-dbx', dest = 'dbx',
                        help = 'initialize dbx with update from FILE', metavar = 'FILE')
    pgroup.add_argument('--add-dbx', dest = 'add_dbx', action = 'append',
                        help = 'append dbx update from FILE', metavar = 'FILE')
    pgroup.add_argument('--add-dbx-cert', dest = 'dbx_cert',  action = 'append', nargs = 2,
                        help = 'add x509 cert to dbx, loaded in pem or der format ' +
                        'from FILE and with owner GUID, can be specified multiple times',
                        metavar = ('GUID', 'FILE'))
    pgroup.add_argument('--add-dbx-hash', dest = 'dbx_hash',  action = 'append', nargs = 2,
                        help = 'add sha256 HASH to dbx, with owner GUID, ' +
                        'can be specified multiple times',
                        metavar = ('GUID', 'HASH'))
    pgroup.add_argument('--add-mok', '--add-mok-cert', dest = 'mok', action = 'append', nargs = 2,
                        help = 'add x509 cert to MokList, loaded in pem or der format ' +
                        'from FILE and with owner GUID, can be specified multiple times',
                        metavar = ('GUID', 'FILE'))
    pgroup.add_argument('--add-mok-hash', dest = 'mok_hash',  action = 'append', nargs = 2,
                        help = 'add sha256 HASH to MokList, with owner GUID, ' +
                        'can be specified multiple times',
                        metavar = ('GUID', 'HASH'))

    pgroup = parser.add_argument_group('Secure boot convinience shortcuts')
    pgroup.add_argument('--enroll-redhat', dest = 'redhat',
                        action = 'store_true', default = False,
                        help = 'enroll default certificates for redhat platform')
    pgroup.add_argument('--enroll-cert', dest = 'enroll_cert',
                        help = 'enroll using specified certificate', metavar = "CERT")
    pgroup.add_argument('--enroll-generate', dest = 'enroll_generate',
                        help = 'enroll using generated cert with given common name', metavar = "CN")
    pgroup.add_argument('--enroll-mgmt', dest = 'enroll_mgmt',
                        action = 'store_true', default = False,
                        help = 'enroll using external management guid')
    pgroup.add_argument('--no-microsoft', dest = 'microsoft',
                        action = 'store_false', default = True,
                        help = 'same as "--microsoft-db none"')
    pgroup.add_argument('--microsoft-db', '--profile', dest = 'ms_db',
                        choices = [ 'none',
                                    'win11', 'win23', 'win+rom23',
                                    'uefi11', 'uefi23', 'uefi+rom23',
                                    'redhat', 'redhat+rom',
                                    'all' ],
                        default = 'all',
                        help = 'choose microsoft db keys to enroll')
    pgroup.add_argument('--microsoft-kek', dest = 'ms_kek',
                        choices = [ 'none', '2011', '2023', 'all' ], default = 'all',
                        help = 'choose microsoft KEK keys to enroll')
    pgroup.add_argument('--distro-keys', dest = 'distro', type = str, action = 'append',
                        help = 'add ca keys for DISTRO', metavar = 'DISTRO')
    pgroup.add_argument('--distro-list', dest = 'distrolist',
                        action = 'store_true', default = False,
                        help = 'list known distros')
    pgroup.add_argument('--sb', '--secure-boot', dest = 'secureboot',
                        action = 'store_true', default = False,
                        help = 'enable secure boot mode')

    pgroup = parser.add_argument_group('Print options')
    pgroup.add_argument('-p', '--print', dest = 'print',
                        action = 'store_true', default = False,
                        help = 'print varstore')
    pgroup.add_argument('-v', '--verbose', dest = 'verbose',
                        action = 'store_true', default = False,
                        help = 'print varstore verbosely')
    pgroup.add_argument('--hashes', dest = 'hashes',
                        action = 'store_true', default = False,
                        help = 'print signature db sha256 hashes')
    pgroup.add_argument('-x', '--hexdump', dest = 'hexdump',
                        action = 'store_true', default = False,
                        help = 'print variable hexdumps')

    pgroup = parser.add_argument_group('Output options')
    pgroup.add_argument('-o', '--output', dest = 'output', type = str,
                        help = 'write edk2 or aws vars to FILE, using the same format '
                        'the --input FILE has.', metavar = 'FILE')
    pgroup.add_argument('--output-aws', dest = 'output_aws', type = str,
                        help = 'write aws vars to FILE', metavar = 'FILE')
    pgroup.add_argument('--output-json', dest = 'output_json', type = str,
                        help = 'write json dump to FILE', metavar = 'FILE')
    pgroup.add_argument('--output-auth', dest = 'output_auth', type = str,
                        help = 'write *.auth files to DIR', metavar = 'DIR')
    options = parser.parse_args()

    logging.basicConfig(format = '%(levelname)s: %(message)s',
                        level = getattr(logging, options.loglevel.upper()))

    varstore = None
    varlist = efivar.EfiVarList()

    if options.distrolist:
        certs.list_distros()
        sys.exit(0)

    if options.inplace:
        if options.input:
            logging.error("using --inplace is incompatible with --input")
            sys.exit(1)
        if options.output:
            logging.error("using --inplace is incompatible with --output")
            sys.exit(1)
        options.input = options.inplace
        options.output = options.inplace

    if options.input:
        varstore = autodetect.open_varstore(options.input)
        if varstore is None:
            logging.error("unknown input file format")
            sys.exit(1)
        varlist = varstore.get_varlist()

    if options.extract:
        for (key, item) in varlist.items():
            sigdb = item.sigdb
            if sigdb:
                sigdb.extract_certs(key)

    if options.delete:
        for name in options.delete:
            varlist.delete(name)

    if options.set_true:
        for item in options.set_true:
            varlist.set_bool(item, True)

    if options.set_false:
        for item in options.set_false:
            varlist.set_bool(item, False)

    if options.set_boot_uri:
        items = options.set_boot_uri.split('/')
        title = 'netboot ' + items[-1]
        bpath = devpath.DevicePath.uri(options.set_boot_uri)
        varlist.set_boot_entry(0x99, title, bpath)
        varlist.set_boot_next(0x99)

    if options.append_boot_filepath:
        for item in options.append_boot_filepath:
            strings = item.replace('/', '\\').split(' ')
            filepath = strings[0]
            if len(strings) > 1:
                optdata = bytes(ucs16.from_string(strings[1]))
            else:
                optdata = None
            items = filepath.split('\\')
            title = 'file ' + items[-1]
            bpath = devpath.DevicePath.filepath(filepath)
            index = varlist.add_boot_entry(title, bpath, optdata)
            varlist.append_boot_order(index)

    if options.set_shim_debug:
        varlist.set_uint32('SHIM_DEBUG', 1)

    if options.set_shim_verbose:
        varlist.set_uint32('SHIM_VERBOSE', 1)

    if options.set_fallback_verbose:
        varlist.set_uint32('FALLBACK_VERBOSE', 1)

    if options.set_fallback_no_reboot:
        varlist.set_uint32('FB_NO_REBOOT', 1)

    if options.sbatlevel:
        varlist.set_from_file('SbatLevel', options.sbatlevel)

    if options.set_json:
        with open(options.set_json, "r", encoding = 'utf-8') as f:
            l = json.loads(f.read(), object_hook = efijson.efi_decode)
        for (key, item) in l.items():
            logging.info('set variable %s from %s', key, options.set_json)
            varlist[key] = item

    if not options.microsoft:
        options.ms_db = 'none'

    if options.redhat:
        varlist.enroll_platform_redhat(ms_kek = options.ms_kek)
        varlist.add_microsoft_keys(ms_db = options.ms_db)

    if options.enroll_cert:
        varlist.enroll_platform_with_cert(options.enroll_cert,
                                          ms_kek = options.ms_kek)
        varlist.add_microsoft_keys(ms_db = options.ms_db)

    if options.enroll_generate:
        varlist.enroll_platform_generate(options.enroll_generate,
                                         ms_kek = options.ms_kek)
        varlist.add_microsoft_keys(ms_db = options.ms_db)

    if options.enroll_mgmt:
        varlist.enroll_managed_platform(ms_kek = options.ms_kek)
        varlist.add_microsoft_keys(ms_db = options.ms_db)

    if options.distro:
        for item in options.distro:
            varlist.add_distro_keys(item)

    if options.pk:
        varlist.add_cert('PK', options.pk[0], options.pk[1], True)

    if options.kek:
        for item in options.kek:
            varlist.add_cert('KEK', item[0], item[1])

    if options.db:
        for item in options.db:
            varlist.add_cert('db', item[0], item[1])

    if options.db_hash:
        for item in options.db_hash:
            varlist.add_hash('db', item[0], item[1])

    if options.dbx:
        logging.info('reading dbx update from %s', options.dbx)
        with open(options.dbx, 'rb') as f:
            dbxupdate = f.read()
        varlist['dbx'] = efivar.EfiVar(ucs16.from_string('dbx'),
                                       authdata = dbxupdate)

    if options.add_dbx:
        if not varlist.get('dbx'):
            varlist.add_dummy_dbx(guids.OvmfEnrollDefaultKeys)
        for item in options.add_dbx:
            logging.info('adding dbx update from %s', item)
            with open(item, 'rb') as f:
                dbxupdate = f.read()
            update = efivar.EfiVar(ucs16.from_string('dbx'),
                                   authdata = dbxupdate)
            varlist['dbx'].sigdb_merge(update)

    if options.dbx_cert:
        for item in options.dbx_cert:
            varlist.add_cert('dbx', item[0], item[1])

    if options.dbx_hash:
        for item in options.dbx_hash:
            varlist.add_hash('dbx', item[0], item[1])

    if options.mok:
        for item in options.mok:
            varlist.add_cert('MokList', item[0], item[1])

    if options.mok_hash:
        for item in options.mok_hash:
            varlist.add_hash('MokList', item[0], item[1])

    if options.secureboot:
        varlist.enable_secureboot()

    if options.print:
        if options.verbose:
            varlist.print_normal(options.hexdump, options.hashes)
        else:
            varlist.print_compact()

    if options.output:
        if varstore:
            varstore.write_varstore(options.output, varlist)
        else:
            logging.error("no input file specified (needed as edk2 varstore template)")
            sys.exit(1)

    if options.output_aws:
        if options.output_aws == "-":
            print(aws.AwsVarStore.base64_varstore(varlist).decode())
        else:
            aws.AwsVarStore.write_varstore(options.output_aws, varlist)

    if options.output_json:
        if options.output_json == "-":
            print(jstore.JsonVarStore.json_varstore(varlist))
        else:
            jstore.JsonVarStore.write_varstore(options.output_json, varlist)

    if options.output_auth:
        authfiles.AuthFilesVarStore.write_varstore(options.output_auth, varlist)

    return 0

if __name__ == '__main__':
    sys.exit(main())
